@@ -1,6 +1,6 @@
 use crate::{
 	lua::{self, *},
-	userdata::{Angle, Vector},
+	userdata::{Angle, Vector}
 };
 
 use super::LUA_SHARED_RAW;
@@ -12,22 +12,14 @@ macro_rules! dyn_symbols {
 		$vis:vis extern $abi:literal fn $name:ident ( $($arg:ident : $argty:ty),* $(,)? ) -> $ret:ty; $($rest:tt)*
 	) => {
 		$(#[$outer])*
-		#[allow(non_upper_case_globals)]
-		pub static $name: Lazy<extern $abi fn( $($arg: $argty),* ) -> $ret> = Lazy::new(|| unsafe {
-			std::mem::transmute( LUA_SHARED_RAW.get::<extern $abi fn($($argty),*) -> $ret>( stringify!($name).as_bytes() ).unwrap() )
-		});
-		dyn_symbols!( $($rest)* );
-	};
-
-	(
-		$(#[$outer:meta])*
-		$vis:vis extern $abi:literal fn $name:ident <$generic:ident>( $($arg:ident : $argty:ty),* $(,)? ) -> $ret:ty; $($rest:tt)*
-	) => {
-		$(#[$outer])*
-		#[allow(non_upper_case_globals)]
-		pub static $name: Lazy<extern $abi fn( $($arg: $argty),* ) -> $ret> = Lazy::new(|| unsafe {
-			std::mem::transmute( LUA_SHARED_RAW.get::<extern $abi fn($($argty),*) -> $ret>( stringify!($name).as_bytes() ).unwrap() )
-		});
+		#[allow(non_snake_case)]
+		$vis fn $name( $($arg: $argty),* ) -> $ret {
+			unsafe {
+				let f = LUA_SHARED_RAW.get::<extern $abi fn($($argty),*) -> $ret>(stringify!($name).as_bytes())
+					.expect(concat!("Couldn't get extern function: ", stringify!($name)));
+				f( $($arg),* )
+			}
+		}
 		dyn_symbols!( $($rest)* );
 	};
 
@@ -43,7 +35,7 @@ macro_rules! dyn_symbols {
 		dyn_symbols!( $($rest)* );
 	};
 
-	() => ()
+	() => ();
 }
 
 macro_rules! lua_macros {
@@ -58,7 +50,7 @@ macro_rules! lua_macros {
 		lua_macros!( $($rest)* );
 	};
 
-	() => ()
+	() => ();
 }
 
 // Create Lazy cells that'll find the functions at runtime when called.
@@ -68,7 +60,6 @@ macro_rules! lua_macros {
 // Loading functions
 dyn_symbols! {
 	/// Function used by [luaL_loadbuffer].
-	///
 	pub extern "C" fn luaL_loadbufferx(
 		l: LuaState,
 		code: LuaString,
@@ -87,7 +78,7 @@ dyn_symbols! {
 	) -> c_int;
 
 	/// Loads a Lua chunk.
-	/// If there are no errors, lua_load pushes the compiled chunk as a Lua function on top of the stack.
+	/// If there are no errors, this pushes the compiled chunk as a Lua function on top of the stack.
 	/// Otherwise, it pushes an error message.
 	/// # Parameters
 	/// * `l` - Lua state,
@@ -126,9 +117,9 @@ dyn_symbols! {
 
 	/// Loads a file as a Lua chunk.
 	/// This function uses [lua_load] to load the chunk in the file named ``filename``.
-	/// If filename is None, then it loads from the standard input.
+	/// If filename is [std::ptr::null_mut()], then it loads from the standard input.
 	/// The first line in the file is ignored if it starts with a # (shebang)
-	pub extern "C" fn luaL_loadfile(l: LuaState, filename: Option<LuaString>) -> c_int;
+	pub extern "C" fn luaL_loadfile(l: LuaState, filename: LuaString) -> c_int;
 
 	/// Same as how [lua_loadx] is to [lua_load].
 	/// You should probably use [luaL_loadfile] instead.
@@ -174,7 +165,7 @@ dyn_symbols! {
 
 	/// Calls the C function func in protected mode.
 	/// ``func`` starts with only one element in its stack, a light userdata containing ud.
-	/// In case of errors, this returns the same error codes as lua_pcall, plus the error object on the top of the stack.
+	/// In case of errors, this returns the same error codes as [lua_pcall], plus the error object on the top of the stack.
 	/// Otherwise, it returns zero, and does not change the stack.
 	/// All values returned by func are discarded.
 	pub extern "C" fn lua_cpcall(l: LuaState, func: LuaCFunction, userdata: *mut c_void) -> c_int;
@@ -239,13 +230,13 @@ dyn_symbols! {
 	pub extern "C" fn lua_getmetatable(l: LuaState, idx: c_int) -> c_int;
 
 	/// Pushes onto the stack the value t\[k\], where t is the value at ``idx``.
-	/// As in Lua, this function may trigger a metamethod for the "index" event (see §2.8).
+	/// As in Lua, this function may trigger a metamethod for the "index" event.
 	pub extern "C" fn lua_getfield(l: LuaState, idx: c_int, key: LuaString) -> ();
 }
 
 // Non-stack getters
 dyn_symbols! {
-	/// Returns the type of the value in the given acceptable index, or LUA_TNONE for a non-valid index (that is, an index to an "empty" stack position).
+	/// Returns the type of the value in the given acceptable index, or [TNONE] for a non-valid index (that is, an index to an "empty" stack position).
 	/// The types returned by lua_type are coded by the following constants:
 	/// [TNIL], [TNUMBER], [TBOOLEAN], [TSTRING], [TTABLE], [TFUNCTION], [TUSERDATA], [TTHREAD], and [TLIGHTUSERDATA].
 	pub extern "C" fn lua_type(l: LuaState, idx: c_int) -> c_int;
@@ -257,11 +248,11 @@ dyn_symbols! {
 	// Type conversion getters
 
 	/// Converts the Lua value at the given index to a C string.
-	/// If len is not 0, it also sets *len with the string length.
-	/// The Lua value must be a string or a number; otherwise, the function returns a nullptr.
+	/// If len is not None, it also sets *len with the string length.
+	/// The Lua value must be a string or a number; otherwise, the function returns a [std::ptr::null()].
 	/// If the value is a number, then lua_tolstring also changes the actual value in the stack to a string.
 	/// (This change confuses lua_next when lua_tolstring is applied to keys during a table traversal.)
-	pub extern "C" fn lua_tolstring(l: LuaState, ind: c_int, size: SizeT) -> Option<LuaString>;
+	pub extern "C" fn lua_tolstring(l: LuaState, ind: c_int, len: *mut SizeT) -> LuaString;
 
 	/// Converts the Lua value at the given acceptable index to a C boolean value (0 or 1).
 	/// Like all tests in Lua, lua_toboolean returns 1 for any Lua value different from false and nil; otherwise returning 0.
@@ -276,11 +267,11 @@ dyn_symbols! {
 	/// #[gmod_open]
 	/// fn entry(l: LuaState) -> i32 {
 	/// 	lua_getglobal(l, cstr!("CurTime"));
-	/// 	let curtime = lua_tocfunction(l, -1).unwrap();
+	/// 	let curtime = lua_tocfunction(l, -1);
 	/// 	0
 	/// }
 	/// ```
-	pub extern "C" fn lua_tocfunction(l: LuaState, idx: c_int) -> Option<LuaCFunction>;
+	pub extern "C" fn lua_tocfunction(l: LuaState, idx: c_int) -> LuaCFunction;
 
 	/// Converts the Lua value at the given acceptable index to the signed integral type [LuaInteger].
 	/// The Lua value must be a number or a string convertible to a number; otherwise, this returns 0.
@@ -291,22 +282,22 @@ dyn_symbols! {
 	/// The Lua value must be a number or a string convertible to a number; otherwise, this returns 0.
 	pub extern "C" fn lua_tonumber(l: LuaState, idx: c_int) -> LuaNumber;
 
-	/// Converts the value at the given acceptable index to a generic C pointer (void*).
+	/// Converts the value at the given acceptable index to a generic C pointer *mut [c_void].
 	/// The value can be a userdata, a table, a thread, or a function; otherwise this returns None.
 	/// Different objects will give different pointers.
 	/// There is no way to convert the pointer back to its original value.
-	pub extern "C" fn lua_topointer(l: LuaState, idx: c_int) -> Option<*mut c_void>;
+	pub extern "C" fn lua_topointer(l: LuaState, idx: c_int) -> *mut c_void;
 
-	/// Converts the value at the given acceptable index to a Lua thread (represented as lua_State*).
+	/// Converts the value at the given acceptable index to a Lua thread (represented as [LuaState]).
 	/// This value must be a thread; otherwise, the function returns None.
-	pub extern "C" fn lua_tothread(l: LuaState, idx: c_int) -> Option<LuaState>;
+	pub extern "C" fn lua_tothread(l: LuaState, idx: c_int) -> LuaState;
 
 	/// Returns the value at the given index assuming it is a userdata.
 	/// # Returns
 	/// If the value at the given acceptable index is a full userdata, returns its block address.
 	/// If the value is a light userdata, returns its pointer.
-	/// Otherwise, returns None.
-	pub extern "C" fn lua_touserdata(l: LuaState, idx: c_int) -> Option<*mut c_void>;
+	/// Otherwise, returns [std::ptr::null_mut()].
+	pub extern "C" fn lua_touserdata(l: LuaState, idx: c_int) -> *mut c_void;
 }
 
 dyn_symbols! {
@@ -349,6 +340,8 @@ dyn_symbols! {
 	pub extern "C" fn lua_pushthread(l: LuaState) -> c_int;
 
 	/// Pushes a formatted [LuaString] to the stack
+	/// Note this is not a direct function but instead a Lazy pointer to a function.
+	/// This is because variadic functions are not yet supported in Rust, besides through external functions and pointers to them.
 	pub extern "C" fn lua_pushfstring(l: LuaState, fmt: LuaString, ...) -> LuaString;
 
 	/// Pushes a number with value ``n`` onto the stack.
@@ -364,8 +357,8 @@ dyn_symbols! {
 	pub extern "C" fn luaL_checknumber(l: LuaState, narg: c_int) -> LuaNumber;
 
 	/// Checks whether the function argument ``narg`` is a string and returns this string.
-	/// If len is not 0 fills *len with the string's length.
-	pub extern "C" fn luaL_checklstring(l: LuaState, narg: c_int, len: SizeT) -> LuaString;
+	/// If len is not [std::ptr::null_mut()] fills *len with the string's length.
+	pub extern "C" fn luaL_checklstring(l: LuaState, narg: c_int, len: *mut SizeT) -> LuaString;
 
 	/// Checks whether the function has an argument of any type (including nil) at position narg.
 	pub extern "C" fn luaL_checkany(l: LuaState, narg: c_int) -> ();
@@ -383,8 +376,8 @@ dyn_symbols! {
 	/// Creates a new Lua state.
 	/// This calls [lua_newstate] with an allocator based on the standard C realloc function and then sets a panic function (see lua_atpanic) that prints an error message to the standard error output in case of fatal errors.
 	/// # Returns
-	/// The newly created [LuaState], or None if the allocation failed (due to memory).
-	pub extern "C" fn luaL_newstate() -> Option<LuaState>;
+	/// The newly created [LuaState], or [std::ptr::null_mut()] if the allocation failed (due to memory).
+	pub extern "C" fn luaL_newstate() -> LuaState;
 
 	/// Creates a new, independent state.
 	/// Note you might be looking for [luaL_newstate], which has no parameters
@@ -392,7 +385,7 @@ dyn_symbols! {
 	/// The argument f is the allocator function;
 	/// Lua does all memory allocation for this state through this function.
 	/// The second argument, ud, is an opaque pointer that Lua simply passes to the allocator in every call.
-	pub extern "C" fn lua_newstate(f: LuaAlloc, ud: *mut c_void) -> Option<LuaState>;
+	pub extern "C" fn lua_newstate(f: LuaAlloc, ud: *mut c_void) -> LuaState;
 
 	/// Creates a new empty table and pushes it onto the stack.
 	/// The new table has space pre-allocated for ``narr`` array elements and ``nrec`` non-array elements.
@@ -413,11 +406,11 @@ dyn_symbols! {
 	/// This is a C API extension to allow control of the VM from "C"
 	/// # Parameters
 	/// * `l` - Lua state
-	/// * `idx` - Stack index of the function to set the mode of. None to set the mode of the entirety of luajit.
+	/// * `idx` - Stack index of the function to set the mode of. `0` to set the mode of the entirety of luajit.
 	/// * `mode` - The mode to set, 'or'ed with a flag from [lua::jit]
 	/// # Returns
 	/// 1 for success, 0 for failure.
-	pub extern "C" fn luaJIT_setmode(l: LuaState, idx: Option<c_int>, jit_mode: c_int) -> c_int;
+	pub extern "C" fn luaJIT_setmode(l: LuaState, idx: c_int, jit_mode: c_int) -> c_int;
 }
 
 // Coroutines
@@ -464,6 +457,8 @@ dyn_symbols! {
 	/// Raises an error.
 	/// The error message format is given by fmt plus any extra arguments, following the same rules of [lua_pushfstring].
 	/// It also adds at the beginning of the message the file name and the line number where the error occurred, if this information is available.
+	/// Note this is not a direct function but instead a Lazy pointer to a function.
+	/// This is because variadic functions are not yet supported in Rust, besides through external functions and pointers to them.
 	pub extern "C" fn luaL_error(l: LuaState, fmt: LuaString, ...) -> !;
 
 	/// Raises an error with the following message, where func is retrieved from the call stack:
@@ -502,10 +497,10 @@ dyn_symbols! {
 	/// Opens all of the standard libraries for a lua state
 	pub extern "C" fn luaL_openlibs(l: LuaState) -> ();
 	/// Internally called by luaL_register, opens given list of LuaRegs with number of functions provided explicitly
-	pub extern "C" fn luaL_openlib(l: LuaState, libname: LuaString, l: *const LuaReg, nup: c_int) -> ();
+	pub extern "C" fn luaL_openlib(l: LuaState, libname: LuaString, reg: *const LuaReg, nup: c_int) -> ();
 
-	/// Registers a ``reg`` of functions onto the Lua State's _G\[libname\].
-	/// For example you could set libname to cstr!("math") to add functions onto the ``math`` table or create it if it does not exist.
+	/// Registers a ``reg`` of functions onto the LuaState's _G\[libname\].
+	/// For example you could set libname to [cstr]!("math") to add functions onto the ``math`` table or create it if it does not exist.
 	///
 	/// When called with libname as std::ptr::null(), it simply registers all functions in the list ``lib`` into the table on the top of the stack.
 	/// # Example
@@ -531,18 +526,18 @@ dyn_symbols! {
 dyn_symbols! {
 	/// Creates and returns a reference, in the table at index t, for the object at the top of the stack (and pops the object).
 	/// A reference is a unique integer key.
-	/// As long as you do not manually add integer keys into table t, luaL_ref ensures the uniqueness of the key it returns.
-	/// You can retrieve an object referred by reference r by calling lua_rawgeti(L, t, r).
+	/// As long as you do not manually add integer keys into table t, [luaL_ref] ensures the uniqueness of the key it returns.
+	/// You can retrieve an object referred by reference r by calling [lua_rawgeti](L, t, r).
 	/// Function luaL_unref frees a reference and its associated object.
 	///
-	/// If the object at the top of the stack is nil, luaL_ref returns the constant LUA_REFNIL.
-	/// The constant LUA_NOREF is guaranteed to be different from any reference returned by luaL_ref.
+	/// If the object at the top of the stack is nil, luaL_ref returns the constant [REFNIL].
+	/// The constant [NOREF] is guaranteed to be different from any reference returned by this.
 	pub extern "C" fn luaL_ref(l: LuaState, t: c_int) -> c_int;
 
-	/// Releases reference ref from the table at index t (see luaL_ref).
+	/// Releases reference ref from the table at index t (see [luaL_ref]).
 	/// The entry is removed from the table, so that the referred object can be collected.
 	/// The reference ref is also freed to be used again.
-	/// If ref is LUA_NOREF or LUA_REFNIL, this does nothing.
+	/// If ref is [NOREF] or [REFNIL], this does nothing.
 	pub extern "C" fn luaL_unref(l: LuaState, t: c_int, r: c_int) -> ();
 }
 
@@ -553,7 +548,7 @@ dyn_symbols! {
 	pub extern "C" fn luaL_newmetatable(l: LuaState, tname: LuaString) -> c_int;
 
 	/// Creates a metatable with type and typeid
-	/// Same as luaL_newmetatable, but also sets the MetaName and MetaID fields of the metatable
+	/// Same as [luaL_newmetatable], but also sets the MetaName and MetaID fields of the metatable
 	/// # Parameters
 	/// * `l` - LuaState
 	/// * `tname` - TypeName to be added to the metatable
@@ -572,39 +567,39 @@ dyn_symbols! {
 	pub extern "C" fn luaL_optinteger(l: LuaState, narg: c_int, d: LuaInteger) -> c_int;
 
 	/// If the function argument narg is a string, returns this string.
-	/// If this argument is absent or is nil, returns d. Otherwise, raises an error.
+	/// If this argument is absent or is nil, returns ``default``. Otherwise, raises an error.
 	///
-	/// If ``sz`` is not 0, fills the position *``sz`` with the results's length.
-	pub extern "C" fn luaL_optlstring(l: LuaState, arg: c_int, d: LuaString, sz: SizeT)
+	/// If ``len`` is not nullptr, fills the position *``len`` with the results's length.
+	pub extern "C" fn luaL_optlstring(l: LuaState, arg: c_int, default: LuaString, len: *mut SizeT)
 		-> LuaString;
 
 	/// If the function argument ``arg`` is a number, returns this number.
-	/// If this argument is absent or is nil, returns ``d``. Otherwise, raises an error.
-	pub extern "C" fn luaL_optnumber(l: LuaState, arg: c_int, d: LuaNumber) -> LuaNumber;
+	/// If this argument is absent or is nil, returns ``default``. Otherwise, raises an error.
+	pub extern "C" fn luaL_optnumber(l: LuaState, arg: c_int, default: LuaNumber) -> LuaNumber;
 }
 
 dyn_symbols! {
 	// x / ref functions
-	/// Converts the Lua value at the given index to the signed integral type lua_Integer.
+	/// Converts the Lua value at the given index to the signed integral type [LuaInteger].
 	/// The Lua value must be an integer, or a number or string convertible to an integer; otherwise, this returns 0.
-	/// If ``isnum`` is not None, its referent is assigned a boolean value that indicates whether the operation succeeded.
-	pub extern "C" fn lua_tointegerx(l: LuaState, index: c_int, isnum: Option<*mut c_int>) -> LuaInteger;
+	/// If ``isnum`` is not [std::ptr::null_mut()], its referent is assigned a boolean value that indicates whether the operation succeeded.
+	pub extern "C" fn lua_tointegerx(l: LuaState, index: c_int, isnum: *mut c_int) -> LuaInteger;
 
 
-	/// Converts the Lua value at the given index to a LuaNumber (f64).
+	/// Converts the Lua value at the given index to a [LuaNumber].
 	/// The Lua value must be a number or a string convertible to a number; otherwise, this returns 0.
-	/// If ``isnum`` is not None, its referent is assigned a boolean value that indicates whether the operation succeeded.
-	pub extern "C" fn lua_tonumberx(l: LuaState, index: c_int, isnum: Option<*mut c_int>) -> LuaNumber;
+	/// If ``isnum`` is not [std::ptr::null_mut()], its referent is assigned a boolean value that indicates whether the operation succeeded.
+	pub extern "C" fn lua_tonumberx(l: LuaState, index: c_int, isnum: *mut c_int) -> LuaNumber;
 }
 
 dyn_symbols! {
 	/// Creates and pushes a traceback of the stack L1.
-	/// If msg is not None it is appended at the beginning of the traceback.
+	/// If msg is not [std::ptr::null_mut()] it is appended at the beginning of the traceback.
 	/// The level parameter tells at which level to start the traceback.
 	pub extern "C" fn luaL_traceback(
 		l: LuaState,
 		state1: LuaState,
-		msg: Option<LuaString>,
+		msg: LuaString,
 		level: c_int,
 	) -> ();
 
@@ -812,7 +807,7 @@ dyn_symbols! {
 	pub extern "C" fn lua_setupvalue(l: LuaState, fidx: c_int, idx: c_int) -> LuaString;
 
 	/// Sets the value of a local variable of a given activation record.
-	/// Parameters ar and n are as in lua_getlocal (see lua_getlocal).
+	/// Parameters ar and n are as in [lua_getlocal].
 	/// lua_setlocal assigns the value at the top of the stack to the variable and returns its name.
 	/// It also pops the value from the stack.
 	pub extern "C" fn lua_setlocal(l: LuaState, ar: *mut LuaDebug, n: c_int) -> LuaString;
@@ -830,7 +825,7 @@ dyn_symbols! {
 
 dyn_symbols! {
 	/// Returns an unique identifier for the upvalue numbered n from the closure at index funcindex.
-	/// Parameters funcindex and n are as in the lua_getupvalue (see lua_getupvalue) (but n cannot be greater than the number of upvalues).
+	/// Parameters funcindex and n are as in the [lua_getupvalue] (but n cannot be greater than the number of upvalues).
 	/// These unique identifiers allow a program to check whether different closures share upvalues.
 	/// Lua closures that share an upvalue (that is, that access a same external local variable) will return identical ids for those upvalue indices.
 	pub extern "C" fn lua_upvalueid(l: LuaState, fidx: c_int, n: c_int) -> *mut c_void;
@@ -845,7 +840,7 @@ dyn_symbols! {
 	/// This function does not allocate any space; the buffer must be declared as a variable.
 	pub extern "C" fn luaL_buffinit(l: LuaState, b: *mut LuaBuffer) -> ();
 
-	/// Returns an address to a space of size $crate::lua::BUFFERSIZE where you can copy a string to be added to buffer [LuaBuffer] `b`.
+	/// Returns an address to a space of size [$crate::lua::BUFFERSIZE] where you can copy a string to be added to buffer [LuaBuffer] `b`.
 	/// After copying the string into this space you must call luaL_addsize with the size of the string to actually add it to the buffer.
 	pub extern "C" fn luaL_prepbuffer(b: *mut LuaBuffer) -> *mut c_char;
 
@@ -924,10 +919,11 @@ lua_macros! {
 		lua_pushcclosure(l, fnc, 0);
 	};
 
-	/// Equivalent to lua_tolstring with len equal to 0
-	/// This may return None if the value at ``idx`` is not a string or a number, use [luaL_optstring] instead if you do not desire an Option<> or unwrap when you are absolutely sure of the type.
-	pub fn lua_tostring(l: LuaState, idx: c_int) -> Option<LuaString> {
-		lua_tolstring(l, idx, 0)
+	/// Equivalent to ``lua_tolstring(l, idx, [std::ptr::null_mut()])``
+	/// This may return None if the value at ``idx`` is not a string or a number.
+	/// You should use [luaL_optstring] instead if you are unsure of the value, or [luaL_checkstring] for function arguments.
+	pub fn lua_tostring(l: LuaState, idx: c_int) -> LuaString {
+		lua_tolstring(l, idx, std::ptr::null_mut())
 	};
 
 	/// Starts and resumes a coroutine in a given thread
@@ -986,7 +982,7 @@ lua_macros! {
 	/// Returns if the code was successfully executed
 	/// Error will be left on the stack if the code failed to execute
 	pub fn luaL_dofile(l: LuaState, filename: LuaString) -> bool {
-		luaL_loadfile(l, Some(filename)) == 0 || lua_pcall(l, 0, lua::MULTRET, 0) == 0
+		luaL_loadfile(l, filename) == 0 || lua_pcall(l, 0, lua::MULTRET, 0) == 0
 	};
 
 	/// Returns value at [crate::lua::REGISTRYINDEX] with name 'name'
@@ -1008,13 +1004,13 @@ lua_macros! {
 
 	/// Asserts that a string argument exists at index 'i'
 	pub fn luaL_checkstring(l: LuaState, i: c_int) -> LuaString {
-		luaL_checklstring(l, i, 0)
+		luaL_checklstring(l, i, std::ptr::null_mut())
 	};
 
 	/// Like lua_tostring or luaL_checkstring, but instead of returning an invalid string / erroring,
 	/// It returns the given `default` string.
 	pub fn luaL_optstring(l: LuaState, i: c_int, default: LuaString) -> LuaString {
-		luaL_optlstring(l, i, default, 0)
+		luaL_optlstring(l, i, default, std::ptr::null_mut())
 	};
 
 	/// Sets the C function ``f`` as the value of global name ``name``.
@@ -1032,14 +1028,19 @@ lua_macros! {
 
 // Userdata helpers
 lua_macros! {
-	pub fn luaL_checkvector(l: LuaState, narg: c_int) -> Vector {
-		unsafe { *( (*luaL_checkudata(l, narg, cstr!("Vector"))).data as *mut _) }
+	#[cfg(feature = "userdata")]
+	/// Returns a [Vector] on the stack at index ``idx``, or if there is no such value, throws a lua argument error.
+	pub fn luaL_checkvector(l: LuaState, idx: c_int) -> Vector {
+		unsafe { *( (*luaL_checkudata(l, idx, cstr!("Vector"))).data as *mut _) }
 	};
 
-	pub fn luaL_checkangle(l: LuaState, narg: c_int) -> crate::userdata::Angle {
-		unsafe { *( (*luaL_checkudata(l, narg, cstr!("Angle"))).data as *mut _) }
+	#[cfg(feature = "userdata")]
+	/// Returns an [Angle] on the stack at index ``idx``, or if there is no such value, throws a lua argument error.
+	pub fn luaL_checkangle(l: LuaState, idx: c_int) -> crate::userdata::Angle {
+		unsafe { *( (*luaL_checkudata(l, idx, cstr!("Angle"))).data as *mut _) }
 	};
 }
+
 /// Pushes a vector onto the stack
 /// # Example
 /// Creates a LuaCFunction that will take three number arguments and return a glua Vector type.
@@ -1055,6 +1056,7 @@ lua_macros! {
 ///     1
 /// }
 /// ```
+#[cfg(feature = "userdata")]
 pub fn lua_pushvector(l: LuaState, v: Vector) {
 	let ptr = lua_newuserdata(l, std::mem::size_of::<Userdata>());
 
@@ -1074,6 +1076,7 @@ pub fn lua_pushvector(l: LuaState, v: Vector) {
 }
 
 /// Pushes an angle onto the stack.
+#[cfg(feature = "userdata")]
 pub fn lua_pushangle(l: LuaState, v: Angle) {
 	let ptr = lua_newuserdata(l, std::mem::size_of::<Userdata>());
 
@@ -1091,8 +1094,8 @@ pub fn lua_pushangle(l: LuaState, v: Angle) {
 
 #[inline(always)]
 #[allow(non_snake_case)]
-/// Tries to see if the given value at index ``arg`` is nil or none, if so, returns ``default`` value.
-/// Otherwise, runs ``func``, passing the lua state and arg indent and returns the value of that
+/// Tries to see if the given value at index ``arg`` is nil or none, if so, returns ``default`` value.  
+/// Otherwise, runs ``func``, passing the lua state and arg indent and returns the value of that  
 /// # Returns
 /// Type ``T`` either from the function or default value.
 pub fn luaL_opt<T, F: Fn(LuaState, c_int) -> T>(l: LuaState, arg: c_int, default: T, func: F) -> T {
@@ -1103,7 +1106,7 @@ pub fn luaL_opt<T, F: Fn(LuaState, c_int) -> T>(l: LuaState, arg: c_int, default
 	}
 }
 
-/// This function works like luaL_checkudata, except that, when the test fails, it returns None instead of throwing an error.
+/// This function works like [luaL_checkudata], except that, when the test fails, it returns None instead of throwing an error.  
 /// Adapted from Lua 5.3, note this does not actually exist in gluajit
 #[allow(non_snake_case)]
 pub fn luaL_testudata(l: LuaState, arg: c_int, tname: LuaString) -> Option<*mut super::Userdata> {
@@ -1111,7 +1114,7 @@ pub fn luaL_testudata(l: LuaState, arg: c_int, tname: LuaString) -> Option<*mut
 		lua_getmetatable(l, arg); // Object metatable
 		luaL_getmetatable(l, tname); // Desired global metatable
 		if lua_rawequal(l, -1, -2) == 1 {
-			return lua_touserdata(l, arg).map(|ud| ud as *mut super::Userdata);
+			return Some(lua_touserdata(l, arg) as *mut super::Userdata);
 		}
 	}
 	None
@@ -1119,7 +1122,16 @@ pub fn luaL_testudata(l: LuaState, arg: c_int, tname: LuaString) -> Option<*mut
 
 #[inline(always)]
 #[allow(non_snake_case)]
-pub fn lua_tovector<'a>(l: LuaState, i: c_int) -> Option<&'a mut Vector> {
-	luaL_testudata(l, i, cstr!("Vector"))
-		.map(|x: *mut Userdata| unsafe { &mut *(x as *mut Vector) })
+#[cfg(feature = "userdata")]
+/// Returns a [Vector] from the stack at index ``i``.
+pub fn lua_tovector<'a>(l: LuaState, i: c_int) -> Option<Vector> {
+	luaL_testudata(l, i, cstr!("Vector")).map(|x: *mut Userdata| unsafe { *(x as *mut Vector) })
+}
+
+#[inline(always)]
+#[allow(non_snake_case)]
+#[cfg(feature = "userdata")]
+/// Returns an [Angle] from the stack at index ``i``.
+pub fn lua_toangle(l: LuaState, i: c_int) -> Option<Angle> {
+	luaL_testudata(l, i, cstr!("Angle")).map(|x: *mut Userdata| unsafe { *(x as *mut Angle) })
 }
